# This file was auto-generated by Fern from our API Definition.

from .api import (
    ITIN,
    SSN,
    AccountType,
    Address,
    Attachments,
    BankAccountId,
    BankAccountRequest,
    BankAccountResponse,
    BankAddress,
    BankLookupResponse,
    BankStatus,
    BankType,
    BirthDate,
    BusinessProfileRequest,
    BusinessProfileResponse,
    BusinessType,
    CardBrand,
    CardId,
    CardRequest,
    CardResponse,
    CardType,
    CheckId,
    CheckRequest,
    CheckResponse,
    CounterpartyResponse,
    CreateVendorRequest,
    CustomId,
    CustomPaymentMethodRequest,
    CustomPaymentMethodResponse,
    DocumentResponse,
    Ein,
    EmailLogResponse,
    EmailOcrRequest,
    EmailProviderRequest,
    EmailProviderResponse,
    EmailSenderProvider,
    EmailSenderRequest,
    EmailSenderResponse,
    EntityId,
    EntityRequest,
    EntityResponse,
    EntityStatus,
    EntityUpdateRequest,
    FullName,
    IndividualGovernmentID,
    IndividualProfileRequest,
    IndividualProfileResponse,
    InvoiceId,
    InvoiceRequest,
    InvoiceResponse,
    InvoiceStatus,
    OcrMailbox,
    OCRResponse,
    OrganizationId,
    OrganizationRequest,
    OrganizationResponse,
    PaymentMethodId,
    PaymentMethodRequest,
    PaymentMethodResponse,
    PaymentMethodSchemaField,
    PaymentMethodSchemaFieldType,
    PaymentMethodSchemaId,
    PaymentMethodSchemaRequest,
    PaymentMethodSchemaResponse,
    PaymentMethodsRequest,
    PaymentMethodsResponse,
    PaymentMethodType,
    PaymentRailMarkup,
    PaymentRailMarkupType,
    PaymentRailRequest,
    PaymentRailResponse,
    PhoneNumber,
    ProfileRequest,
    ProfileResponse,
    RepresentativeId,
    RepresentativeRequest,
    RepresentativeResponse,
    Responsibilities,
    TaxID,
    TransactionResponse,
    TransactionStatus,
    UnauthorizedError,
    bank_lookup,
    commons,
    counterparty,
    entity,
    invoice,
    ocr,
    organization,
    payment_method,
    payment_method_schema,
    representative,
)
from .environment import MercoaEnvironment

__all__ = [
    "AccountType",
    "Address",
    "Attachments",
    "BankAccountId",
    "BankAccountRequest",
    "BankAccountResponse",
    "BankAddress",
    "BankLookupResponse",
    "BankStatus",
    "BankType",
    "BirthDate",
    "BusinessProfileRequest",
    "BusinessProfileResponse",
    "BusinessType",
    "CardBrand",
    "CardId",
    "CardRequest",
    "CardResponse",
    "CardType",
    "CheckId",
    "CheckRequest",
    "CheckResponse",
    "CounterpartyResponse",
    "CreateVendorRequest",
    "CustomId",
    "CustomPaymentMethodRequest",
    "CustomPaymentMethodResponse",
    "DocumentResponse",
    "Ein",
    "EmailLogResponse",
    "EmailOcrRequest",
    "EmailProviderRequest",
    "EmailProviderResponse",
    "EmailSenderProvider",
    "EmailSenderRequest",
    "EmailSenderResponse",
    "EntityId",
    "EntityRequest",
    "EntityResponse",
    "EntityStatus",
    "EntityUpdateRequest",
    "FullName",
    "ITIN",
    "IndividualGovernmentID",
    "IndividualProfileRequest",
    "IndividualProfileResponse",
    "InvoiceId",
    "InvoiceRequest",
    "InvoiceResponse",
    "InvoiceStatus",
    "MercoaEnvironment",
    "OCRResponse",
    "OcrMailbox",
    "OrganizationId",
    "OrganizationRequest",
    "OrganizationResponse",
    "PaymentMethodId",
    "PaymentMethodRequest",
    "PaymentMethodResponse",
    "PaymentMethodSchemaField",
    "PaymentMethodSchemaFieldType",
    "PaymentMethodSchemaId",
    "PaymentMethodSchemaRequest",
    "PaymentMethodSchemaResponse",
    "PaymentMethodType",
    "PaymentMethodsRequest",
    "PaymentMethodsResponse",
    "PaymentRailMarkup",
    "PaymentRailMarkupType",
    "PaymentRailRequest",
    "PaymentRailResponse",
    "PhoneNumber",
    "ProfileRequest",
    "ProfileResponse",
    "RepresentativeId",
    "RepresentativeRequest",
    "RepresentativeResponse",
    "Responsibilities",
    "SSN",
    "TaxID",
    "TransactionResponse",
    "TransactionStatus",
    "UnauthorizedError",
    "bank_lookup",
    "commons",
    "counterparty",
    "entity",
    "invoice",
    "ocr",
    "organization",
    "payment_method",
    "payment_method_schema",
    "representative",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ....environment import MercoaEnvironment
from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_headers import remove_none_from_headers
from ..entity.types.entity_id import EntityId
from .types.representative_id import RepresentativeId
from .types.representative_request import RepresentativeRequest
from .types.representative_response import RepresentativeResponse


class RepresentativeClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def get_all(self, entity_id: EntityId) -> typing.List[RepresentativeResponse]:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"entity/{entity_id}/representatives"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[RepresentativeResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, entity_id: EntityId, *, request: RepresentativeRequest) -> RepresentativeResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"entity/{entity_id}/representative"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RepresentativeResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, entity_id: EntityId, representative_id: RepresentativeId) -> RepresentativeResponse:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._environment.value}/", f"entity/{entity_id}/representative/{representative_id}"
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RepresentativeResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, entity_id: EntityId, representative_id: RepresentativeId) -> None:
        _response = httpx.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._environment.value}/", f"entity/{entity_id}/representative/{representative_id}"
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRepresentativeClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def get_all(self, entity_id: EntityId) -> typing.List[RepresentativeResponse]:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"entity/{entity_id}/representatives"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[RepresentativeResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, entity_id: EntityId, *, request: RepresentativeRequest) -> RepresentativeResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"entity/{entity_id}/representative"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RepresentativeResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, entity_id: EntityId, representative_id: RepresentativeId) -> RepresentativeResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(
                    f"{self._environment.value}/", f"entity/{entity_id}/representative/{representative_id}"
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RepresentativeResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, entity_id: EntityId, representative_id: RepresentativeId) -> None:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "DELETE",
                urllib.parse.urljoin(
                    f"{self._environment.value}/", f"entity/{entity_id}/representative/{representative_id}"
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ....environment import MercoaEnvironment
from ...core.api_error import ApiError
from ...core.remove_none_from_headers import remove_none_from_headers
from .types.bank_lookup_response import BankLookupResponse


class BankLookupClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def find(self, *, routing_number: str) -> BankLookupResponse:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", "bankLookup"),
            params={"routingNumber": routing_number},
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BankLookupResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBankLookupClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def find(self, *, routing_number: str) -> BankLookupResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", "bankLookup"),
                params={"routingNumber": routing_number},
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(BankLookupResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime


class BankAddress(pydantic.BaseModel):
    address: str
    city: str
    state: str
    postal_code: str = pydantic.Field(alias="postalCode")
    postal_code_extension: str = pydantic.Field(alias="postalCodeExtension")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

from backports.cached_property import cached_property

from ..environment import MercoaEnvironment
from .resources.bank_lookup.client import AsyncBankLookupClient, BankLookupClient
from .resources.counterparty.client import AsyncCounterpartyClient, CounterpartyClient
from .resources.entity.client import AsyncEntityClient, EntityClient
from .resources.invoice.client import AsyncInvoiceClient, InvoiceClient
from .resources.ocr.client import AsyncOcrClient, OcrClient
from .resources.organization.client import AsyncOrganizationClient, OrganizationClient
from .resources.payment_method.client import AsyncPaymentMethodClient, PaymentMethodClient
from .resources.payment_method_schema.client import AsyncPaymentMethodSchemaClient, PaymentMethodSchemaClient
from .resources.representative.client import AsyncRepresentativeClient, RepresentativeClient


class Mercoa:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    @cached_property
    def bank_lookup(self) -> BankLookupClient:
        return BankLookupClient(environment=self._environment, token=self._token)

    @cached_property
    def counterparty(self) -> CounterpartyClient:
        return CounterpartyClient(environment=self._environment, token=self._token)

    @cached_property
    def entity(self) -> EntityClient:
        return EntityClient(environment=self._environment, token=self._token)

    @cached_property
    def invoice(self) -> InvoiceClient:
        return InvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def ocr(self) -> OcrClient:
        return OcrClient(environment=self._environment, token=self._token)

    @cached_property
    def organization(self) -> OrganizationClient:
        return OrganizationClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method_schema(self) -> PaymentMethodSchemaClient:
        return PaymentMethodSchemaClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method(self) -> PaymentMethodClient:
        return PaymentMethodClient(environment=self._environment, token=self._token)

    @cached_property
    def representative(self) -> RepresentativeClient:
        return RepresentativeClient(environment=self._environment, token=self._token)


class AsyncMercoa:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    @cached_property
    def bank_lookup(self) -> AsyncBankLookupClient:
        return AsyncBankLookupClient(environment=self._environment, token=self._token)

    @cached_property
    def counterparty(self) -> AsyncCounterpartyClient:
        return AsyncCounterpartyClient(environment=self._environment, token=self._token)

    @cached_property
    def entity(self) -> AsyncEntityClient:
        return AsyncEntityClient(environment=self._environment, token=self._token)

    @cached_property
    def invoice(self) -> AsyncInvoiceClient:
        return AsyncInvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def ocr(self) -> AsyncOcrClient:
        return AsyncOcrClient(environment=self._environment, token=self._token)

    @cached_property
    def organization(self) -> AsyncOrganizationClient:
        return AsyncOrganizationClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method_schema(self) -> AsyncPaymentMethodSchemaClient:
        return AsyncPaymentMethodSchemaClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method(self) -> AsyncPaymentMethodClient:
        return AsyncPaymentMethodClient(environment=self._environment, token=self._token)

    @cached_property
    def representative(self) -> AsyncRepresentativeClient:
        return AsyncRepresentativeClient(environment=self._environment, token=self._token)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from .bank_account_request import BankAccountRequest
from .card_request import CardRequest
from .check_request import CheckRequest
from .custom_payment_method_request import CustomPaymentMethodRequest
from .payment_method_type import PaymentMethodType


class PaymentMethodRequest(pydantic.BaseModel):
    type: PaymentMethodType
    bank_account: typing.Optional[BankAccountRequest] = pydantic.Field(alias="bankAccount")
    check: typing.Optional[CheckRequest]
    card: typing.Optional[CardRequest]
    custom: typing.Optional[CustomPaymentMethodRequest]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateVendorRequest,
    DocumentResponse,
    InvoiceId,
    InvoiceRequest,
    InvoiceResponse,
    InvoiceStatus,
    TransactionResponse,
    TransactionStatus,
)

__all__ = [
    "CreateVendorRequest",
    "DocumentResponse",
    "InvoiceId",
    "InvoiceRequest",
    "InvoiceResponse",
    "InvoiceStatus",
    "TransactionResponse",
    "TransactionStatus",
]



# This file was auto-generated by Fern from our API Definition.

from .create_vendor_request import CreateVendorRequest
from .document_response import DocumentResponse
from .invoice_id import InvoiceId
from .invoice_request import InvoiceRequest
from .invoice_response import InvoiceResponse
from .invoice_status import InvoiceStatus
from .transaction_response import TransactionResponse
from .transaction_status import TransactionStatus

__all__ = [
    "CreateVendorRequest",
    "DocumentResponse",
    "InvoiceId",
    "InvoiceRequest",
    "InvoiceResponse",
    "InvoiceStatus",
    "TransactionResponse",
    "TransactionStatus",
]
